# (c) Copyright 2020-2021. CodeWeavers, Inc.

import cxlog
import cxobjc
import cxproduct
import cxurlget
import cxutils
import proxyinfo

import json


class WebToken(cxobjc.Proxy):
    pass

def get_token():
    config = cxproduct.get_config()
    return config['CrossOver'].get('Token', None)

@cxobjc.method(WebToken, 'createToken_password_')
def create_token(login, password):
    url = 'https://www.codeweavers.com/bin/rest/create_token'

    # Get an md5 of the password.
    md5_hasher = cxutils.md5_hasher()
    md5_hasher.update(cxutils.string_to_utf8(password))
    md5_password = md5_hasher.hexdigest()

    token = None

    try:
        proxyinfo.install_default_opener()

        data = cxurlget.fetch_content(url, timeout=5,
            data={'username': login, 'password': md5_password})
        token = json.loads(cxutils.string_to_unicode(data))
    except cxurlget.UrlError as e:
        # Errors in communication with the server
        cxlog.log(e.reason)
        return False

    if token is not None:
        config = cxproduct.get_config()
        wconfig = config.get_save_config()
        wconfig.lock_file()
        wconfig['CrossOver']['Token'] = token
        wconfig.save_and_unlock_file()

        return True

    return False
